<?php

namespace Drupal\mdp\Plugin\Field\FieldType;

use Drupal\Core\Field\Attribute\FieldType;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\TypedData\MapDataDefinition;

/**
 * Plugin implementation of the 'mdp' field type.
 */
#[FieldType(
  id: "field_mdp",
  label: new TranslatableMarkup('Multi Date Picker (MDP)'),
  description: [
    new TranslatableMarkup('This field allows pick between multiple dates.'),
    new TranslatableMarkup('Ideal to schedule events, for example between 2 and 5 dates.'),
    new TranslatableMarkup('If pick a date this will be validate how a unique date, if pick only two dates these will be take how a range of dates and will validate all dates between them including the picked dates and if pick more three dates these will be validate each one.'),
  ],
  category: "date_time",
  default_widget: "field_mdp_widget",
  default_formatter: "field_mdp_formatter",
)]
class MDPItem extends FieldItemBase {

  /**
   * Value for the 'mdp_type' setting: store only a dates.
   */
  const MDP_ONLY_DATE = 'onlyDates';

  /**
   * Value for the 'mdp_type' setting: store dates with hours.
   */
  //const MDP_DATE_WITH_HOURS = 'datesWithHours';

  /**
   * Value for the 'mdp_type' setting: store a date and time.
   * This config will be create in the future.
   */
  //const MDP_WITH_HOURS = 'withHours';

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return [
      'columns' => [
        'value' => [
          'type' => 'blob',
          'size' => 'big',
          'serialize' => TRUE,
        ]
      ]
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultStorageSettings() {
    return [
      'mdp_type' => self::MDP_ONLY_DATE,
    ] + parent::defaultStorageSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function storageSettingsForm(array &$form, FormStateInterface $form_state, $has_data) {
    $element = [];
    $element['mdp_type'] = [
      '#type' => 'select',
      '#title' => $this->t('MDP type'),
      '#default_value' => $this->getSetting('mdp_type'),
      '#options' => [
        self::MDP_ONLY_DATE => $this->t('Only Dates'),
        //self::MDP_DATE_WITH_HOURS => $this->t('Dates with hours')
      ],
      '#disabled' => $has_data,
      '#description' => $this->t("The hours will be from 00:00 to 23:59 for each day."),
      // '#description' => $this->t("If <strong></strong>only dates</strong> is checked the hours will be from 00:00 to 23:59 for each day.<br>
      // If <strong>dates with hours</strong> is checked the hours will be the selected."),
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultFieldSettings() {
    return [
      'separator' => ', ',
    ] + parent::defaultStorageSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function fieldSettingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::fieldSettingsForm($form, $form_state);

    $element['separator'] = [
      '#type' => 'select',
      '#title' => $this->t('Separator'),
      '#options' => [
        ', ' => $this->t('Comma'),
        '; ' => $this->t('Semicolon'),
        ' ' => $this->t('Space'),
        ' - ' => $this->t('Dash'),
        chr(8201) => $this->t('Thin space'),
      ],
      '#default_value' => $this->getSetting('separator'),
    ];
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $value = $this->getValue();
    return $value === NULL || $value === '';
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['value'] = MapDataDefinition::create()
      ->setLabel(new TranslatableMarkup('Dates'))
      ->setDescription(t('Dates values.'))
      ->setRequired(TRUE);

      return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public function setValue($values, $notify = TRUE) {
    $values = [static::mainPropertyName() => $values];
    parent::setValue($values, $notify);
  }
}
